
import java.util.zip.*;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.ObjectInputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;
import java.nio.charset.Charset;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Scanner;


import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;


public class CapiscoTopicExpansion extends HttpServlet {
    //private static final long serialVersionUID = 1L;

    private static final int BUFFER_SIZE = 4096;

    private static HashMap<String,List<String>> TopicsToSynonyms = new HashMap<String,List<String>>();

    private static void init_lookup(InputStream is)
    {
	try {
	    BufferedInputStream bis = new BufferedInputStream(is);
	    ObjectInputStream ois = new ObjectInputStream(bis);
	    TopicsToSynonyms = (HashMap<String, List<String>>) ois.readObject();
	    bis.close();
	    ois.close();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    private static void init_lookup(String filename)
    {
	try {
	    FileInputStream fis = new FileInputStream(filename);
	    init_lookup(fis);
	    fis.close();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    private static void lookup_loop()
    {
	Scanner scan = new Scanner(System.in);
	System.out.println("Press enter to start:");
	String wait = scan.nextLine();
	
	
	System.out.println("Enter words to test (enter '.quit' to quit program)");
	
	boolean quit = false;
	while (!quit) {
	    String test_word = scan.next();
	    if (test_word.equals(".quit")) {
		quit = true;
	    }
	    else {
		List<String> synonyms = TopicsToSynonyms.get(test_word);

		System.out.println("Lookup ["+test_word+"] = " + synonyms);
	    }
	}
	scan.close();
	System.out.println("Program ended.");
    }

    /**
     * @see Servlet#init(ServletConfig)
     */
    public void init(ServletConfig config) throws ServletException 
    {
	super.init(config);

	try {
	    InputStream is = getServletContext().getResourceAsStream("/WEB-INF/topics-to-synonyms.ser");

	    init_lookup(is);
	    is.close();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }

    /**
     * @see HttpServlet#doGet(HttpServletRequest request, HttpServletResponse response)
     */
    protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
	PrintWriter pw = response.getWriter();
	
	String cgi_topic = request.getParameter("topic");
	
	if (cgi_topic != null) {
	    response.setContentType("application/json");
	    
	    String topic = cgi_topic;
	    List<String> synonyms = TopicsToSynonyms.get(topic);

	    pw.append("[");
	    for (int i=0; i<synonyms.size(); i++) {
		String synonym = synonyms.get(i);
		if (i>0) {
		    pw.append(",");
		}
		pw.append("'" + synonym + "'");

	    }

	    pw.append("]");

	}
	else {

	    pw.append("General Info: Number of topics for expansion = " + TopicsToSynonyms.size());
	    
	}
	//pw.close();
	
    }


    /**
     * @see HttpServlet#doPost(HttpServletRequest request, HttpServletResponse response)
     */
    protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
	doGet(request, response);
    }


    public static void main(String[] args) {

	String filename = "topics-to-synonyms.ser";

	init_lookup(filename);
	
	lookup_loop();

    }
}

